//AssigningTool.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

import swarm.collections.ListImpl;
import swarm.collections.ListIndex;
import swarm.collections.ArrayImpl;

/**
 * A tool to choose the unit to which we have to send an order
 *
 * @author Pietro Terna
 */
public class AssigningTool extends SwarmObjectImpl
{
    /** the list of the normal units existing in our environment*/
    public ListImpl unitList;
    /** the iterator of unitList */
    public ListIndex unitListIndex;
    /** the temporary address of an existing unit */
    public Unit aUnit;
    /** the activity space of the first unit */
    public UnitActivitySpace unitActivitySpace;
    /** the min level of inter-visibility; below this level two units cannot
     *  exchange products; this value is not used in assignment coming from 
     *  OrderGenerator; this value can be different from one model to another: that
     *  of the sending unit is used */
    public double interVisibilityMinLevel;

    /** the numner of our layer and the number of layers (or different models) 
     *  in this run */
    public int layer, layerNumber;

    public AssigningTool (Zone aZone, int ln
			  /*ListImpl ul, ListIndex uli, 
			    UnitActivitySpace uas, double iv*/)
    {
	super(aZone);

	// the number of layers (differen models) in this run
	layerNumber=ln;
    }

    /** assignment between two units */
    public Order assign(Unit sendingUnit, Order firstOrder)
    {
	int i, ii, la, layerSendingUnit;
	boolean unitNotFound, applied;
	
	// data of the sending unit
	layerSendingUnit=((ESFrameModelSwarm) sendingUnit.getUnitModel()).getLayer();
	unitActivitySpace = (UnitActivitySpace) InformationRepository.
	    getUnitActivitySpaceFromModel(layerSendingUnit);
	interVisibilityMinLevel =  InformationRepository.getInterVisibilityMinLevel(layerSendingUnit);

	unitNotFound=true;
	for (la=0;la<layerNumber;la++)
	    {
		unitList= (ListImpl) InformationRepository.
		    getUnitListFromReorderedModelSequence(la);
		unitListIndex= (ListIndex) InformationRepository.
		    getUnitListIndexFromReorderedModelSequence(la);

		unitListIndex.setOffset(0);

		for (i=0;i<unitList.getCount() && unitNotFound ;i++)
		    {
			aUnit=(Unit)unitListIndex.get();
			if (firstOrder.getNextStep() ==	aUnit.getProductionPhase())
			    {
				// no visibility check or same unit 
				if (interVisibilityMinLevel == 0 || sendingUnit == aUnit)
				    unitNotFound = false;
				// second unit belongs to the first unit area
				else if (aUnit.getXPos()<unitActivitySpace.getXSize() &&
					 aUnit.getYPos()<unitActivitySpace.getYSize())
 				    //units seeing each other above the min visibility level
				    if(unitActivitySpace.interVisibility(sendingUnit, aUnit) >= 
				       interVisibilityMinLevel)unitNotFound = false;
			    }
			unitListIndex.next();
		    }
	    }

	if(! unitNotFound)
	    {
		aUnit.setWaitingList(firstOrder);
		return (Order) null;
	    }
	else return firstOrder;

    }

    /** assignment from the order generator */
    public Order assign(Order firstOrder)
    {
	int i, ii, la;
	boolean unitNotFound, applied;

	unitNotFound=true;
	for (la=0;la<layerNumber;la++)
	    {
		unitList= (ListImpl) InformationRepository.
		    getUnitListFromReorderedModelSequence(la);
		unitListIndex= (ListIndex) InformationRepository.
		    getUnitListIndexFromReorderedModelSequence(la);

		unitListIndex.setOffset(0);

		for (i=0;i<unitList.getCount() && unitNotFound ;i++)
		    {
			aUnit=(Unit)unitListIndex.get();
			if (firstOrder.getNextStep() ==	aUnit.getProductionPhase())
			    unitNotFound=false;
			
			unitListIndex.next();
		    }
	    }
	if(! unitNotFound)
	    {
		aUnit.setWaitingList(firstOrder);
		return (Order) null;
	    }
	

	return firstOrder;

	}
}
