// InformationRepository.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

import swarm.collections.ListImpl;
import swarm.collections.ListIndex;
import swarm.collections.ArrayImpl;

/** 
 * A static class used to distribute information among models
 *
 * @author Pietro Terna
 */
public class InformationRepository
{
    /** the arrays of unitList, unitListIndex, unitActivitySpace */
    public static ArrayImpl unitListArray, unitListIndexArray, unitActivitySpaceArray;
    /** the vector of interVisibilityMinLevel */
    public static double interVisibilityMinLevelVector[];

    /** the vectors with model numbers */
    public static int modelNumbers[], modelNumbersRandomlyReordered[];

    /** the number of layers (models) in this run */
    public static int layerNumber;

    /** the creation of the memory spaces, once in a run, from the first model */
    public static void createArraysAndVectors(Zone aZone, int ln)
    {
	layerNumber=ln;
	unitListArray = new ArrayImpl (aZone, layerNumber);
	unitListIndexArray = new ArrayImpl (aZone, layerNumber);
	unitActivitySpaceArray = new ArrayImpl (aZone, layerNumber);
	interVisibilityMinLevelVector = new double[layerNumber];

	modelNumbers = new int[layerNumber];
	modelNumbersRandomlyReordered = new int[layerNumber];
    }

    /** setting information, for each model */
    public static void setInformation(int layer, ListImpl unitList, 
				      ListIndex unitListIndex, 
				      UnitActivitySpace unitActivitySpace, 
				      double interVisibilityMinLevel)
    {
	unitListArray.atOffset$put(layer, unitList);
	unitListIndexArray.atOffset$put(layer, unitListIndex);
	unitActivitySpaceArray.atOffset$put(layer, unitActivitySpace);
	interVisibilityMinLevelVector[layer]=interVisibilityMinLevel;
    }

    /** shuffling in each cycle, from the first model schedule, the model order */
    public static void setModelSequence()
    {
	int i, ii, ir;

	for (i=0;i<layerNumber;i++)modelNumbers[i]=i;

	for (i=layerNumber-1;i>=0;i--)
	    {
		ir=Globals.env.uniformIntRand.getIntegerWithMin$withMax(0, i);
		modelNumbersRandomlyReordered[i]=modelNumbers[ir];
		for (ii=ir;ii<i;ii++)modelNumbers[ii]=modelNumbers[ii+1];
	    }

    }

    /** getting a specific unitList from random model sequence */
    public static Object getUnitListFromReorderedModelSequence(int level)
    {
	return unitListArray.atOffset(modelNumbersRandomlyReordered[level]);
    }

    /** getting a specific unitListIndex from random model sequence */
    public static Object getUnitListIndexFromReorderedModelSequence(int level)
    {
	return unitListIndexArray.atOffset(modelNumbersRandomlyReordered[level]);
    }

    /** getting a specific unitActivitySpace */
    public static Object getUnitActivitySpaceFromModel(int level)
    {
	return unitActivitySpaceArray.atOffset(level);
    }

    /** getting a specific interVisibilityMinLevel */
    public static double getInterVisibilityMinLevel(int level)
    {
	return interVisibilityMinLevelVector[level];
    }

}
