// Order.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;
import swarm.Selector;

import swarm.collections.ListImpl;
import swarm.collections.ListIndex;

/**
 * The form containing our Order metaphoric object
 *
 * @author Pietro Terna (with Paolo Pelligra and Dario Landini contibutions)
 */
public class Order  extends SwarmObjectImpl implements Cloneable{

    /** the number of steps in the product recipe */
    public int stepNumber;
    /** the 'name' of the recipe (may be used by OrderDistiller, as a way
     *  to identify recipes */
    public String recipeName="noName";
    /** the number of this order */
    public int orderNumber;
   /** the day in which the order as been generated */
    public int startingTimeUnit;
    /** integer vector, containing the recipe steps */
    public int[] orderRecipe;
    /** integer vector, recording the performed phases */
    public int[] orderState;

    /** the model */
    ESFrameModelSwarm eSFrameModelSwarm;

    /** indexes used in a common way in the full version of jES*/
    public int i, j, jj, k;

    /** incoming recipe structure; we make a local copy of it, from r0 which
     *  is in the constructor paramenter list, to r */
    public int[] r;

  /**
   * the constructor for Order
   */
  public Order (Zone aZone, int n, int sd, int sn, int [] r0, 
		ESFrameModelSwarm model) {

     // Call the constructor for the Order parent class
     super(aZone);

     r = new int[r0.length];
     for (i=0;i<r0.length;i++)r[i]=r0[i];

     orderNumber=n;
     stepNumber=sn;
     startingTimeUnit=sd;
     orderRecipe= new int[stepNumber];
     orderState = new int[stepNumber];
     // the model
     eSFrameModelSwarm = model;

     i=0; j=0;

     // copying the recipe
     while (j<stepNumber)
	 {
	     if (r[j]==0){
		 System.out.
		     println("Internal error in order generation: " + 
			     "look at the constructor of Order class.");
		 MyExit.exit(1);
	     }

	     if (r[j]>0) orderRecipe[i]=r[j];

	     i++; j++;
	 }

     // i is now the internal length of vectors, so we redefine the 
     // stepNumber (unnecessary in this version but mandatory in the full jES structure)
     stepNumber=i;

     // announcing the order presence to the console
     if(StartESFrame.verbose)
	 System.out.println("New order (#" + orderNumber + ")");

     // setting the state vector (0=empty; not zero=done)
     if(StartESFrame.verbose)System.out.print("orderState  vector ");
     for (i=1;i<=stepNumber;i++) {
	 orderState[i-1]=0;
	 if(StartESFrame.verbose)System.out.print(orderState[i-1] + " ");
     }
     if(StartESFrame.verbose)System.out.println();

     // announcing the recipe vector
     if(StartESFrame.verbose)
	 {
	 System.out.print("orderRecipe vector ");
	 for (i=1;i<=stepNumber;i++)
	     System.out.print(orderRecipe[i-1] + " ");
	 System.out.println();
     }
  }


  /**
   * production next step in Order (it is in the orderRecipe vector,
   * at the same index value in which we find the first 0 in orderState
   * vector)
   */
    public int getNextStep () {

	int i, nextStep;

	nextStep=0;
	for (i=0;i< stepNumber && nextStep==0;i++)
	    if (orderState[i]==0)nextStep=getOrderRecipe(i);

	// if no 0 position can be found the order is finished
	// in this case we return 0

	return nextStep;
  }

  /**
   * production done step in Order (it is in the orderRecipe vector,
   * at the index value in which we find the first 0 in orderState
   * vector minus 1; if no step has been done, the method returns -999999)
   */
    public int getDoneStep () {

	int i, doneStep, firstZeroPosition;

	firstZeroPosition=-1;
	for (i=0;i< stepNumber && firstZeroPosition==-1;i++)
	    if (orderState[i]==0)firstZeroPosition=i;

	// if no 0 position can be found, the order is finished;
	// in this case we return the last possible step
	if(firstZeroPosition==-1)doneStep=getOrderRecipe(stepNumber-1);

	// if the first 0 is in 0 position, no done step exists, so we return
	// -999999
	else 
	    if(firstZeroPosition==0)doneStep=-999999;
	    else 
		if(orderState[firstZeroPosition-1]==-1)
		    doneStep=-1;  // we are on the border of an 
	                          // unused 'or' branch
		else doneStep=getOrderRecipe(firstZeroPosition-1);

	return doneStep;
  }



    /**
     * Done step in Order
     */
    public void setDoneStep (int u) {

	int i, done;
	if(StartESFrame.verbose)
	    System.out.println("done step in order #"+getOrderNumber());
	done=0;
	for (i=0;i<= stepNumber && done==0;i++)
	    if (orderState[i]==0)
		{orderState[i]=u;
		done=1;
		}
    }

  /**
   * returning the orderNumber of this order
   */
    public int getOrderNumber () {
	return orderNumber;
  }

  /**
   * returning the stepNumber
   */
    public int getStepNumber () {

	return stepNumber;
  }

  /**
   * returning the startingTimeUnit
   */
    public int getStartingTimeUnit() {

	return startingTimeUnit;
  }


   /** evaluate the number of direct steps done */
    public float getNumberOfDirectStepsDone(){
	float globalOrderState, d, dRatio; 
	int i;
 
	globalOrderState=0; d=0; dRatio=0;
	for(i=0; i<orderState.length; i++){ /* not accounting for zero time steps*/
	    if(getOrderState(i)>0 && getOrderRecipe(i)<1000000000)
		globalOrderState++;
	}


	if(StartESFrame.verbose)
	    System.out.println(
			       "in Order # " + getOrderNumber());

	return globalOrderState;
   }


    /** getting the content of orderRecipe[i] */
    public int getOrderRecipe(int i){
	if(i>=stepNumber){
	    System.out.println(
	 "Error in getting a recipe step in an order (max length exceeded)");
	    MyExit.exit(1);
	}
	return orderRecipe[i];
    }

    /** getting the content of orderState[i] */
    public int getOrderState(int i){
	if(i>=stepNumber){
	    System.out.println(
	 "Error in getting a state value in an order (max length exceeded)");
	    MyExit.exit(1);
	}
	return orderState[i];
    }


    /** settinging the name of the recipe */
    public void setRecipeName(String s){
	recipeName=s;
    }

    /** retrieving the name of the recipe */
    public String getRecipeName(){
	return recipeName;
    }


}
