//Recipe.java for jesopenfoundation; reduced commenting the unused features

import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;
/**
 * Recipe.java
 * This class is used to read the recipes and their code from the worksheet.
 * Recipes are converted in the intermediate format of jES and ready to be assigned 
 * to a List

 * @author Cristian Barreca, Elena Bonessa, Antonella Borra, modified by Marco Lamieri, Francesco Merlo, refactored by Francesco Merlo
 */
public class Recipe extends SwarmObjectImpl
{
	String
		semicolon = ";",
		gate = "#",
		p = "p",
		end = "e",
		sec = "s",
		min = "m",
		slash = "/",
		backslash = "\\",
		or = "||",
		computation="c",
		recipeName,
		currentCell,
		timeUnit;

	ExcelReader recipeWorkSheet;

	int
		timeNumber,
		stepNumber,
		recipeCode,
		recipeLength;

	int[]
		steps,
		recipeSteps;

	//CONSTRUCTOR
	public Recipe(Zone aZone)
	{
		super(aZone);
		recipeSteps = new int[0];
	}

	// **************
	// PUBLIC METHODS
	// **************
	
	/** This method is used to store the intermediate format of a recipe in a List. See OrderDistiller.java.
	*/
	public void setRecipeFrom(ExcelReader e)
	{
	        //System.out.println("Getting a new recipe.......");
		recipeWorkSheet = e;
		currentCell="";
		//Checking for a comment in the row
		checkForComment();
		//Getting the recipe name
		recipeName = recipeWorkSheet.getIfString();
		//Getting the recipe code
		recipeCode = recipeWorkSheet.getIfInteger();

		if(StartESFrame.verbose)
			System.out.println("Recipe: reading a new recipe with name '" 
					   + recipeName + "' and code '" +  
					   recipeCode + "'...");

		//Reading the recipe
		while(!currentCell.equals(semicolon))
		{
			//The recipe has...
			if(recipeWorkSheet.checkForLabelCell())
			{
				currentCell = recipeWorkSheet.getStrValue();
				//... a computational step
				if(currentCell.equals(computation))
				    computation(); //see below				
				
				//... a procurement step
				/* not used in jesopenfoundation
				else if(currentCell.equals(p))
				         procurement(); */ //see below

				//... an or step
				/* not used in jesopenfoundation
				else if(currentCell.equals(or))
				         or(); */  //see below

				//... an end step
				/* not used in jesopenfoundation
				else if(currentCell.equals(end))
				         end(); */ //see below

				//This is the end of line
				else if(currentCell.equals(semicolon))
					{System.err.println("End of recipe");
					break;}
				else
				{
					System.err.println("Recipe error: unknown sign '" + 
							   currentCell +"'.");
					MyExit.exit(1);
				}
			}
			else
			{
				//... a simple step
				number(); //see below
			}

			if(StartESFrame.verbose)
			{
				System.out.print("Recipe: intermediate format of this step is '");
				for(int s=0; s<steps.length; s++)
					System.out.print(steps[s] + " ");
				System.out.println("'");
			}
			//Adding the new step (in the intermediate format) to the array 'recipeSteps[]'
			addSteps();

			//currentCell = recipeWorkSheet.getStrValue();
			//System.out.println("Current cell is " + currentCell);
			//recipeWorkSheet.goBack();
		}

		if(StartESFrame.verbose)
		{
			System.out.print("Recipe: the intermediate format of recipe '"+ recipeName +"' is '");
			for(int s=0; s<recipeSteps.length; s++)
				System.out.print(recipeSteps[s] + " ");
			System.out.println("'");
		}
	}
/**Returns the name of the recipe*/
	public String getRecipeName()
	{
		return recipeName;
	}
/**Returns the code of the recipe*/
	public int getRecipeCode()
	{
		return recipeCode;
	}
/**Returns the steps (in the intermediate format) of the recipe*/
	public int[] getRecipeSteps()
	{
		return recipeSteps;
	}
/**Returns the length of the recipe in the intermediate format)*/
	public int getLength()
	{
		return recipeSteps.length;
	}

	// *****************
	// PRIVATE FUNCTIONS
	// *****************
	private void addSteps()
	{
		//Adding the new steps of the recipe
		int[] newRecipeSteps;
		int newRecipeLength = recipeSteps.length + steps.length;
		newRecipeSteps = new int[newRecipeLength];
		System.arraycopy(recipeSteps, 0, newRecipeSteps, 0, recipeSteps.length);
		System.arraycopy(steps, 0, newRecipeSteps, recipeSteps.length, steps.length);
		recipeSteps = new int[newRecipeLength];
		System.arraycopy(newRecipeSteps, 0, recipeSteps, 0, newRecipeLength);
	}

	private int[] procurement()
	{
		//Reading the number of components
		int numberOfStepsForProcurement = recipeWorkSheet.getIfInteger();
		//The array for the step in the intermediate format
		steps = new int[numberOfStepsForProcurement+2];
		//Intermediate code for procurement
		steps[0] = -1;
		//The number of components
		steps[1] = numberOfStepsForProcurement;
		//Storing the endUnit codes
		for(int p=0; p<numberOfStepsForProcurement; p++)
			steps[p+2]= recipeWorkSheet.getIfInteger();

		if(StartESFrame.verbose)
		{
			System.out.print("Recipe: reading PROCUREMENT step 'p " + numberOfStepsForProcurement + " ");
			for(int p=2; p<steps.length; p++)
				System.out.print(steps[p]+" ");
			System.out.println("'");
		}

		return steps;
    }

	private int[] number()
	{
		//Getting the unit code...
		//stepNumber = Integer.parseInt(currentCell);
		stepNumber = recipeWorkSheet.getIfInteger();
		//... the time unit ...
		timeUnit = recipeWorkSheet.getIfString();
		//... and the time number
		timeNumber = recipeWorkSheet.getIfInteger();

		//Checking for a batch step...
		String isBatch = recipeWorkSheet.getStrValue();
		//System.out.println("isBatch is " + isBatch);

			//...is a stand alone batch
			if(isBatch.equals(slash))
			{
				int items = recipeWorkSheet.getIfInteger();
				String isEnd = recipeWorkSheet.getIfString();
				if(!isEnd.equals(end))
				{
					System.err.println("Recipe error: there is no endUnit in " + recipeName);
					MyExit.exit(1);
				}
				int endUnit = recipeWorkSheet.getIfInteger();
				steps = new int[5];
				steps[0] = -2;
				steps[1] = timeNumber;
				steps[2] = items;
				steps[3] = stepNumber;
				steps[4] = endUnit;
				if(StartESFrame.verbose)
					System.out.println("Recipe: reading STAND ALONE BATCH step '" + stepNumber + " " + timeUnit +
						" " + timeNumber + " / " + items + " e " + endUnit +"'");
			}
			//... is a sequential batch
			else if(isBatch.equals(backslash))
			{
				int items = recipeWorkSheet.getIfInteger();
				steps = new int[4];
				steps[0] = -3;
				steps[1] = timeNumber;
				steps[2] = items;
				steps[3] = stepNumber;
				if(StartESFrame.verbose)
					System.out.println("Recipe: reading SEQUENTIAL BATCH step '" + stepNumber + " " + timeUnit +
						" " + timeNumber + " \\ " + items + "'");
			}
			//... it is not a batch
			else
			{
				recipeWorkSheet.goBack();



				if(StartESFrame.verbose)
					System.out.println("Recipe: reading step '" + stepNumber + " " + timeUnit + " " + timeNumber +"'");
				if(timeUnit.equals(min))
					timeNumber = timeNumber*60;
				else if(!timeUnit.equals(sec))
				{
					System.err.println("Recipe: time is not expressed in minutes or seconds. Check the worksheet");
					MyExit.exit(1);
				}
				if(timeNumber==0)
				{
					steps = new int[1];
					steps[0] = 1000000000 + stepNumber;
				}
				else
				{
					steps = new int[timeNumber];
					for(int s=0; s<timeNumber; s++)
						steps[s] = stepNumber;
				}
			}
		return steps;
	}


	private int[] computation()
	{
		// Getting the computation code
		int computationCode = recipeWorkSheet.getIfInteger();
		// Getting the number of matrixes used for computation
		int numberOfMatrixesForComputation = recipeWorkSheet.getIfInteger();
		// Creating an array with the name of matrixes
		int[] matrixesForComputation = new int[numberOfMatrixesForComputation];
		// Getting the matrixes
		for(int m = 0; m < numberOfMatrixesForComputation; m++)
			matrixesForComputation[m] = recipeWorkSheet.getIfInteger();
		// Getting the production step
		int stepNumber = recipeWorkSheet.getIfInteger();
		// Getting the time unit
		String timeUnit = recipeWorkSheet.getIfString();
		//Getting the time number
		int timeNumber = recipeWorkSheet.getIfInteger();
		
		//Storing the matrixes code
		if(StartESFrame.verbose){
			System.out.print("Recipe: reading COMPUTATIONAL step 'c " + computationCode + " " +
				numberOfMatrixesForComputation + " ");
		for(int c=0; c<numberOfMatrixesForComputation; c++)
			System.out.print(matrixesForComputation[c] + " ");
		System.out.println(stepNumber + " " + timeUnit + " " + timeNumber + "'");
		}

		if(timeUnit.equals(min))
			timeNumber = timeNumber*60;

		if(timeNumber==0)
		{
			steps = new int[numberOfMatrixesForComputation + 4];
			steps[0] = 1000000000 + stepNumber;
			steps[1] = -1 * computationCode;
			steps[2] = numberOfMatrixesForComputation;
			for(int m = 0; m < numberOfMatrixesForComputation; m++)
				steps[3+m] = matrixesForComputation[m];
			steps[numberOfMatrixesForComputation + 3] = 1000000000 + stepNumber;
		}
		else
		{
			steps = new int[timeNumber + numberOfMatrixesForComputation + 3];
			for(int s=0; s<timeNumber; s++)
				steps[s] = stepNumber;

			steps[timeNumber] = -1 * computationCode;
			steps[timeNumber+1] = numberOfMatrixesForComputation;
			for(int m = 0; m < numberOfMatrixesForComputation; m++)
				steps[timeNumber+2+m] = matrixesForComputation[m];
			steps[timeNumber + numberOfMatrixesForComputation + 2] = 1000000000 + stepNumber;
		}
		return steps;
	}

	private int[] or()
	{
		steps = new int[2];
		steps[0] = -10;
		steps[1] = recipeWorkSheet.getIfInteger();
		if(StartESFrame.verbose)
			System.out.println("Recipe: reading OR step '|| " + steps[1] + "'");
		return steps;
	}

	private int[] end()
	{
		steps = new int[1];
		steps[0] = recipeWorkSheet.getIfInteger();
		if(StartESFrame.verbose)
			System.out.println("Recipe: reading END step 'e " + steps[0] + "'");
		return steps;
	}

	private void checkForComment()
	{
		if(recipeWorkSheet.checkForLabelCell())
	 	{
			String isGate = recipeWorkSheet.getStrValue();
			if(isGate.equals(gate))
			{
				if(StartESFrame.verbose)
				{
					System.out.println("Recipe: there is a comment ...");
					System.out.print("# ");
				}
				while(!recipeWorkSheet.eol())
				{
					String comment = recipeWorkSheet.getStrValue();
					if(StartESFrame.verbose)
						System.out.print(" " + comment + " |");
				}
				String comment = recipeWorkSheet.getStrValue();
				if(StartESFrame.verbose)
					System.out.println(" " + comment + " #");
				
				//Checking for other lines of comments
				checkForComment();
			}
			else
			{
				recipeWorkSheet.goBack();
			}
		}
	}
}//Recipe.java
